# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =

# Emphasizer
# Nov. 16, 2004
# Revised Feb 16, 2005...
# Copyright (c) 2005, David G. Drumright [ockham]
# New WX version May 9, 2013 ... No internal changes

# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
# Global vars
#-----------------------------------------------------------------------------

global ParmList, TheFig, TheActor, ActorName, FigName

# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =

import wx
import wx.py
import wx.aui

import poser
import random

scene = poser.Scene()

ActorName=''
FigName=''
TheFig=None
TheActor=scene.CurrentActor()
ActorName=TheActor.Name()
try:
    TheFig=TheActor.ItsFigure()
    FigName=TheFig.Name()
except:
    TheFig=None


ParmList={}

# Start wx
man = poser.WxAuiManager()
root = man.GetManagedWindow()

# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =
# Next section is WX
# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =

class App(wx.Frame):

    # - - - - - - - - - - - - - - - - - -

    def __init__(self,parent,title):
       wx.Frame.__init__(self,parent,title=title,size=(700, 440))


       self.Status = self.CreateStatusBar()

       # Start sizers
       ParmsSizer=wx.BoxSizer(wx.VERTICAL)  # left column
       LabelSizer=wx.BoxSizer(wx.VERTICAL)  # mid column
       EntrySizer=wx.BoxSizer(wx.VERTICAL)  # right column
       BottomSizer=wx.BoxSizer(wx.HORIZONTAL) # bottom row
       MainSizer = wx.GridBagSizer(hgap=4,vgap=4)
       MainSizer.Add(ParmsSizer,(0,0),(12,1))
       MainSizer.Add(LabelSizer,(0,1),(12,1))
       MainSizer.Add(EntrySizer,(0,2),(12,1))
       MainSizer.Add(BottomSizer,(13,0),(1,3))

       # Form parm list
       self.ParmBox = wx.ListCtrl(self, size=(200,400),style=wx.LC_REPORT|wx.BORDER_SUNKEN)
       self.ParmBox.InsertColumn(0, 'Parameters')
       self.ParmBox.SetColumnWidth(0,200)
       self.FillParmList()
       ParmsSizer.Add(self.ParmBox)


       # Form the entry boxes
       self.MultLabel = wx.StaticText(self,-1,label='Multiply by',size = (80, 24))
       self.MultEntry = wx.TextCtrl(self, size = (80, 24))
       self.MultEntry.SetValue(str(1.0))
       LabelSizer.Add(self.MultLabel,wx.ALL,5)
       EntrySizer.Add(self.MultEntry,wx.ALL,5)

       self.AddLabel = wx.StaticText(self,-1,label='Offset by',size = (80, 24))
       self.AddEntry = wx.TextCtrl(self,size=(80,24))
       self.AddEntry.SetValue(str(0.0))
       LabelSizer.Add(self.AddLabel,wx.ALL,5)
       EntrySizer.Add(self.AddEntry,wx.ALL,5)

       self.RandLabel = wx.StaticText(self,-1,label='Random range',size = (80, 24))
       self.RandEntry = wx.TextCtrl(self,size=(80,24))
       self.RandEntry.SetValue(str(0.0))
       LabelSizer.Add(self.RandLabel,wx.ALL,5)
       EntrySizer.Add(self.RandEntry,wx.ALL,5)

       self.BiasLabel = wx.StaticText(self,-1,label='Random bias',size = (80, 24))
       self.BiasEntry = wx.TextCtrl(self,size=(80,24))
       self.BiasEntry.SetValue(str(0.0))
       LabelSizer.Add(self.BiasLabel,wx.ALL,5)
       EntrySizer.Add(self.BiasEntry,wx.ALL,5)

       self.SkipLabel = wx.StaticText(self,-1,label='Sparseness (frames)',size = (80, 24))
       self.SkipEntry = wx.TextCtrl(self,size=(80,24))
       self.SkipEntry.SetValue(str(1))
       LabelSizer.Add(self.SkipLabel,wx.ALL,5)
       EntrySizer.Add(self.SkipEntry,wx.ALL,5)

       self.StartFrameLabel = wx.StaticText(self,-1,label='Start frame',size = (80, 24))
       self.StartFrameEntry = wx.TextCtrl(self,size=(80,24))
       self.StartFrameEntry.SetValue(str(1))
       LabelSizer.Add(self.StartFrameLabel,wx.ALL,5)
       EntrySizer.Add(self.StartFrameEntry,wx.ALL,5)

       self.EndFrameLabel = wx.StaticText(self,-1,label='End frame',size = (80, 24))
       self.EndFrameEntry = wx.TextCtrl(self,size=(80,24))
       self.EndFrameEntry.SetValue(str(scene.NumFrames()))
       LabelSizer.Add(self.EndFrameLabel,wx.ALL,5)
       EntrySizer.Add(self.EndFrameEntry,wx.ALL,5)

       self.DropEntry=wx.CheckBox(self, -1 ,'Drop to floor', size=(80,24))
       LabelSizer.Add(self.DropEntry,wx.ALL,5)



       # Buttons on bottom
       self.ButtonSparsify = wx.Button(self,   id=-1, label='Sparsify', size=(80, 32))
       self.ButtonSparsify.Bind(wx.EVT_BUTTON, self.HandleSpars)
       self.ButtonGo       = wx.Button(self,   id=-1, label='Emphasize',size=(80, 32))
       self.ButtonGo.Bind(wx.EVT_BUTTON, self.HandleGo)
       BottomSizer.Add(self.ButtonSparsify,wx.ALL,5)
       BottomSizer.Add(self.ButtonGo,      wx.ALL,5)


       self.SetSizerAndFit(MainSizer)

       self.Show()

       scene.SetEventCallback(self.EventCB) # Start the callback
       # Show status at bottom
       if TheFig==None:
           self.Status.SetStatusText('Actor %s' % ActorName)
       else:
           self.Status.SetStatusText('%s in figure %s' % (ActorName,FigName))


# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =

    def FillParmList(self):
        # Clear then refill the Parm list
        global ParmList, ActorName, TheFig, TheActor
        # Clear then refill the Parm list
        # Check to be sure the selected part is going to be usable for our purpose.
        # If not, go to Ground.

        ParmList={} # clear list
        self.ParmBox.DeleteAllItems() # clear box
        i=0
        for OneParm in TheActor.Parameters():
            if not OneParm.Hidden():
                self.ParmBox.InsertStringItem(index=i,label=OneParm.Name())
                ParmList[i]=OneParm.Name()
                i=i+1

    # - - - - - - - - - - - - - - - - - -
    def get_selected_items(self,list_control):
        # This is peculiar, but seems to be necessary.  Copied from
        # http://ginstrom.com/scribbles/2008/09/14/getting-selected-items-from-a-listctrl-in-wxpython/
        selection = []
        # start at -1 to get the first selected item
        current = -1
        while True:
            next = list_control.GetNextItem(current,wx.LIST_NEXT_ALL,wx.LIST_STATE_SELECTED)
            if next == -1: return selection
            selection.append(next)
            current = next
        # end while

    # - - - - - - - - - - - - - - - - - -
    # Event callback so we can sense different actors
    def EventCB(self, iScene, iEventType):
        global ParmList, TheFig, TheActor, ActorName, FigName
        if( (iEventType & poser.kEventCodeACTORSELECTIONCHANGED) != 0):
            # Redo the parm-finding at the top
            ActorName=''
            FigName=''
            TheFig=None
            TheActor=scene.CurrentActor()
            ActorName=TheActor.Name()
            try:
                TheFig=TheActor.ItsFigure()
                FigName=TheFig.Name()
            except:
                TheFig=None
            # Re-show status after the change
            if TheFig==None:
                self.Status.SetStatusText('Actor %s' % ActorName)
            else:
                self.Status.SetStatusText('%s in figure %s' % (ActorName,FigName))
            self.FillParmList() # redo the parm box

    # - - - - - - - - - - - - - - - - - -


    def HandleGo(self,event):  # just shell for button that needs event
        self.Action()

    # - - - - - - - - - - - - - - - - - -

    def Action(self):
        global ParmList
        StartFrameString = self.StartFrameEntry.GetValue()
        EndFrameString = self.EndFrameEntry.GetValue()
        MultString = self.MultEntry.GetValue()
        AddString = self.AddEntry.GetValue()
        RandString = self.RandEntry.GetValue()
        BiasString = self.BiasEntry.GetValue()
        SkipString = self.SkipEntry.GetValue()

        if StartFrameString:
            StartFrame = int(StartFrameString)
        else:
            StartFrame = 0

        if EndFrameString:
            EndFrame = int(EndFrameString)
        else:
            EndFrame = scene.NumFrames()

        if MultString:
            Multiplier = float(MultString)
        else:
            Multiplier = 1.0  # No change if you miss

        if AddString:
            Adder      = float(AddString)
        else:
            Adder = 0.0  # No change if you miss

        if RandString:
            Rand      = float(RandString)
        else:
            Rand = 0.0  # No change if you miss

        if BiasString:
            Bias      = float(BiasString) * 0.01
        else:
            Bias = 0.0  # No change if you miss

        if SkipString:
            Skip      = int(SkipString)
        else:
            Skip = 1  # Min


        # Get the selections:

        Sels = self.get_selected_items(self.ParmBox)
        if not Sels: return
        for OneSel in Sels:
            OneParmName = ParmList[OneSel]
            try:
                OneParm = TheActor.Parameter(OneParmName)
            except:
                continue
            # Force all keyframes so we don't accidentally continue to end.
            for F in range(scene.NumFrames()):
                scene.SetFrame(F)
                OneParm.AddKeyFrame(F)

            if string.count(string.upper(OneParmName),'SCALE'):
                RealAdder=Adder/100.0
            else:
                RealAdder=Adder

            # Do somewhat different if random is given.
            if Rand > 0.0:
                # If there are existing keyframes except for the first,
                # touch only the existing keyframes.
                # First get the keyframes into an array.
                KeyFrameList=[]
                NKF=StartFrame
                while NKF <= EndFrame:
                    scene.SetFrame(NKF)
                    NKF=OneParm.NextKeyFrame()
                    if NKF==None: break
                    KeyFrameList.append(NKF)

                if len(KeyFrameList)>1: # we seem to have existing KF.
                    for F in KeyFrameList:
                        scene.SetFrame(F)
                        G=random.gauss(0.0,0.5)
                        if   G >  1.0: G =  1.0
                        elif G < -1.0: G = -1.0
                        RandAdd=(Rand+Bias)*G
                        OneParm.SetValue(OneParm.Value()*Multiplier+RealAdder)
                        OneParm.SetValue(OneParm.Value()+RandAdd)

                else: # Using random, but no existing keyframes.  Use the Skip, with slight random on time.
                    for F in range(StartFrame,EndFrame,Skip): # Now change the value at each point
                        scene.SetFrame(F+random.uniform(-2.1,2.1))
                        G=random.gauss(0.0,0.5)
                        if   G >  1.0: G =  1.0
                        elif G < -1.0: G = -1.0
                        RandAdd=(Rand+Bias)*G
                        OneParm.SetValue(OneParm.Value()*Multiplier+RealAdder)
                        OneParm.SetValue(OneParm.Value()+RandAdd)

            else: # No random; just set all frames in the range.
                for F in range(StartFrame,EndFrame): # Now change the value at each point
                    scene.SetFrame(F)
                    G=random.gauss(0.0,0.5)
                    if   G >  1.0: G =  1.0
                    elif G < -1.0: G = -1.0
                    RandAdd=(Rand+Bias)*G
                    OneParm.SetValue(OneParm.Value()*Multiplier+RealAdder)
                    OneParm.SetValue(OneParm.Value()+RandAdd)

        # Get drop to floor
        if self.DropEntry.IsChecked():
            for F in range(scene.NumFrames()):
                scene.SetFrame(F)
                TheFig.DropToFloor()

        scene.SetFrame(0)
        self.Status.SetStatusText('Done with Emphasize.')

    # - - - - - - - - - - - - - - - - - -

    def HandleSpars(self):
        # Go through all keyframes.  Cut out odd-numbered frames that are
        # not "necessary".  Specifically, if the value at the odd-numbered frame is
        # between the values of its neighboring evens, eliminate it.

        # To avoid possible interference between the changes and the actual,
        # build an array of the frames to delete first; then go through and delete them.

        TheActor=TheFig.Actor(ActorName)
        Sels=self.ParmBox.curselection()
        if not Sels:
            self.PartListBox.selection_set(0,END)
            Sels =self.PartListBox.curselection()
        for S in Sels:
            ParmName = self.ParmBox.get(int(S))
            OneParm=TheActor.Parameter(ParmName)
            if OneParm.Hidden(): continue
            Last=scene.NumFrames()-1
            ActualFrames=[]
            NeedsDelete=[]

            # Begin by getting a list of the actual keyframes so we can work
            # on that basis instead of raw frames.  This will make it possible
            # to use the script repeatedly.
            F=1
            while (F != None) and (F < Last):
                scene.SetFrame(F)
                ActualFrames.append(F)
                F=OneParm.NextKeyFrame(F)

            # Now do the real run through these frames.
            for F in ActualFrames:
                scene.SetFrame(F)
                ThisVal=OneParm.Value()

                # Look at its neighbors.  Check to be sure
                # we're not beyond ends.
                try:
                    Prev=OneParm.PrevKeyFrame()
                except:
                    continue
                Next=OneParm.NextKeyFrame()
                if not Next:    break
                if Next >= Last: break

                # After testing, get the values:
                scene.SetFrame(Prev)
                PrevVal=OneParm.Value()
                scene.SetFrame(Next)
                NextVal=OneParm.Value()

                Keep=0 # Var not really used, just makes the Ifs easier to edit.
                if   (ThisVal > PrevVal) and (ThisVal > NextVal): Keep=1  # Local max
                elif (ThisVal < PrevVal) and (ThisVal < NextVal): Keep=1  # Local min
                else:
                    NeedsDelete.append(F)

            # Now go through and use the list. Reminder: at this point
            # in the indents, we are still in the same parameter of the same actor.
            for F in NeedsDelete:
                scene.SetFrame(F)
                OneParm.DeleteKeyFrame(F)

        self.Status.SetStatusText('Done with Sparsify.')

# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

frame = App(None,title='Emphasizer')
frame.Show()

#------------------------------------------------------------
# End wx loop.
#-----------------------------------------------------------------------------
# = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = = =

