#-----------------------------------------------------------------------------
# Emphasizer
# Nov. 16, 2004
# Revised Jan 26, 2005
# Copyright (c) 2005, David G. Drumright [ockham]
#-----------------------------------------------------------------------------

from Tkinter import *
root = Tk()
import random
random.seed()
import poser
scene = poser.Scene()

ActorName=''
TheActor=scene.CurrentActor()
ActorName=TheActor.Name()
try:
    TheFig=TheActor.ItsFigure()
except:
    TheFig=0

#-----------------------------------------------------------------------------
# Global vars
#-----------------------------------------------------------------------------
global ParmList
ParmList=[]

#-----------------------------------------------------------------------------
# Next section is TK
#------------------------------------------------------------

class App:
    def __init__(self, master, textMessage):
        global Go
        Go = 0
        self.master = master
        master.title("Emphasizer on %s" % ActorName)
        self.DropVar=IntVar()
        self.DropVar.set(0)

        self.ParmsFrame=Frame(self.master,borderwidth=2,relief=RIDGE)
        self.EntryFrame=Frame(self.master,borderwidth=2,relief=RIDGE)
        self.ParmsFrame.grid(row=0,column=0,sticky=N+S+E,pady=2)
        self.EntryFrame.grid(row=0,column=1,sticky=N+S+E,pady=2)

        self.ParmLabel=Label(self.ParmsFrame,text="Parameters:").grid(row=0, column=1,sticky=N)
        self.ParmScroll=Scrollbar(self.ParmsFrame, orient=VERTICAL)
        self.ParmScroll.grid(row=1, column=0,sticky=N+S+E)
        self.ParmListBox=Listbox(self.ParmsFrame, height=15, selectmode=MULTIPLE,yscrollcommand=self.ParmScroll.set,exportselection=0)
        self.ParmListBox.grid( row=1, column=1)
        self.ParmScroll["command"]=self.ParmListBox.yview

        self.FillParmList()

        Label(self.EntryFrame,text="Multiply by").grid(row=0, column=0)
        self.MultEntry = Entry(self.EntryFrame,width=8)
        self.MultEntry.grid(row=0,column=1)
        self.MultEntry.insert(0,str(1.0))

        Label(self.EntryFrame,text="Offset by").grid(row=1, column=0)
        self.AddEntry = Entry(self.EntryFrame,width=8)
        self.AddEntry.grid(row=1,column=1)
        self.AddEntry.insert(0,str(0.0))

        Label(self.EntryFrame,text="Random percent").grid(row=2,sticky=E, column=0)
        self.RandEntry = Entry(self.EntryFrame,width=8)
        self.RandEntry.grid(row=2,column=1)
        self.RandEntry.insert(0,str(0.0))

        Label(self.EntryFrame,text="Random bias").grid(row=3,sticky=E, column=0)
        self.BiasEntry = Entry(self.EntryFrame,width=8)
        self.BiasEntry.grid(row=3,column=1)
        self.BiasEntry.insert(0,str(0.0))

        Label(self.EntryFrame,text="Sparseness (frames)").grid(row=4,sticky=E, column=0)
        self.SkipEntry = Entry(self.EntryFrame,width=8)
        self.SkipEntry.grid(row=4,column=1)
        self.SkipEntry.insert(0,str(1))

        Label(self.EntryFrame,text="Start frame").grid(row=5,column=0)
        self.StartFrameEntry=Entry(self.EntryFrame,width=8)
        self.StartFrameEntry.grid(row=5,column=1,sticky=W,pady=3)
        self.StartFrameEntry.insert(0,str(1))

        Label(self.EntryFrame,text="End frame").grid(row=6,column=0,sticky=W,pady=3)
        self.EndFrameEntry=Entry(self.EntryFrame,width=8)
        self.EndFrameEntry.grid(row=6,column=1,sticky=W,pady=3)
        self.EndFrameEntry.insert(0,str(scene.NumFrames()))

        Label(self.EntryFrame,text="Drop to floor").grid(row=7,column=0,sticky=W,pady=3)
        self.DropEntry=Checkbutton(self.EntryFrame,var=self.DropVar)
        self.DropEntry.grid(row=7,column=1,sticky=W,pady=3)

        # Go/nogo buttons.
        self.buttonGo = Button(self.master, text="Go", command=self.handleGo)
        self.buttonGo.grid(row=3, column=0)

        self.buttonCancel = Button(self.master, text="Cancel", command=self.handleCancel)
        self.buttonCancel.grid(row=3, column=2)

    # - - - - - - - - - - - - - - - - - -

    def FillParmList(self):
        # Clear then refill the Parm list
        global ParmList
        global TheFig
        global TheActor
        # Clear then refill the Parm list
        # Check to be sure the selected part is going to be usable for our purpose.
        # If not, go to Ground.

        ParmList =[] # clear
        self.ParmListBox.delete(0,END) # clear
        for OneParm in TheActor.Parameters():
            if not OneParm.Hidden():
                ParmList.append(OneParm.Name())
                self.ParmListBox.insert(END,OneParm.Name())

    # - - - - - - - - - - - - - - - - - -

    def Action(self):
        StartFrameString = self.StartFrameEntry.get()
        EndFrameString = self.EndFrameEntry.get()
        MultString = self.MultEntry.get()
        AddString = self.AddEntry.get()
        RandString = self.RandEntry.get()
        BiasString = self.BiasEntry.get()
        SkipString = self.SkipEntry.get()

        if StartFrameString:
            StartFrame = int(StartFrameString)
        else:
            StartFrame = 0

        if EndFrameString:
            EndFrame = int(EndFrameString)
        else:
            EndFrame = scene.NumFrames()

        if MultString:
            Multiplier = float(MultString)
        else:
            Multiplier = 1.0  # No change if you miss

        if AddString:
            Adder      = float(AddString)
        else:
            Adder = 0.0  # No change if you miss

        if RandString:
            Rand      = float(RandString) * 0.01
        else:
            Rand = 0.0  # No change if you miss

        if BiasString:
            Bias      = float(BiasString) * 0.01
        else:
            Bias = 0.0  # No change if you miss

        if SkipString:
            Skip      = int(SkipString)
        else:
            Skip = 1  # Min


        # Get the selections:
        Sels=self.ParmListBox.curselection()
        if not Sels: return
        for OneSel in Sels:
            OneParmName = ParmList[int(OneSel)]
            try:
                OneParm = TheActor.Parameter(OneParmName)
            except:
                continue
            # Force all keyframes so we don't accidentally continue to end.
            for F in range(scene.NumFrames()):
                scene.SetFrame(F)
                OneParm.AddKeyFrame(F)

            if string.count(string.upper(OneParmName),'SCALE'):
                RealAdder=Adder/100.0
            else:
                RealAdder=Adder

            # Do somewhat different if random is given.
            if Rand > 0.0:
                # If there are existing keyframes except for the first,
                # touch only the existing keyframes.
                # First get the keyframes into an array.
                KeyFrameList=[]
                NKF=StartFrame
                while NKF <= EndFrame:
                    scene.SetFrame(NKF)
                    NKF=OneParm.NextKeyFrame()
                    if NKF==None: break
                    KeyFrameList.append(NKF)

                if len(KeyFrameList)>1: # we seem to have existing KF.
                    for F in KeyFrameList:
                        scene.SetFrame(F)
                        RandAdd=(Rand+Bias)*(random.random()-0.5)
                        OneParm.SetValue(OneParm.Value()*Multiplier+RealAdder)
                        OneParm.SetValue(OneParm.Value()+RandAdd)

                else: # Using random, but no existing keyframes.  Use the Skip.
                    for F in range(StartFrame,EndFrame,Skip): # Now change the value at each point
                        scene.SetFrame(F)
                        RandAdd=(Rand+Bias)*(random.random()-0.5)
                        OneParm.SetValue(OneParm.Value()*Multiplier+RealAdder)
                        OneParm.SetValue(OneParm.Value()+RandAdd)

            else: # No random; just set all frames in the range.
                for F in range(StartFrame,EndFrame): # Now change the value at each point
                    scene.SetFrame(F)
                    RandAdd=(Rand+Bias)*(random.random()-0.5)
                    OneParm.SetValue(OneParm.Value()*Multiplier+RealAdder)
                    OneParm.SetValue(OneParm.Value()+RandAdd)

        if self.DropVar.get():
            for F in range(scene.NumFrames()):
                scene.SetFrame(F)
                TheFig.DropToFloor()

        scene.SetFrame(0)

    # - - - - - - - - - - - - - - - - - -

    def Update(self):  # Share the wealth
        if scene: scene.ProcessSomeEvents(1)
        root.lift()
        root.after(250, self.Update)

    # - - - - - - - - - - - - - - - - - -

    def handleGo(self):
        self.Action()
        self.master.destroy()

    # - - - - - - - - - - - - - - - - - -

    def handleCancel(self):
        self.master.destroy()

#------------------------------------------------------------
# Activate the loop
#------------------------------------------------------------

app = App(root, '')
app.Update()
root.mainloop()

#------------------------------------------------------------
# End TK loop.
#-----------------------------------------------------------------------------
